//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "parallel/ParallelProblemStatBase.h"
#include "parallel/MeshDistributor.h"
#include "parallel/MpiHelper.h"
#include "Global.h"

namespace AMDiS {

  void ParallelProblemStatBase::buildAfterCoarsen(AdaptInfo *adaptInfo, Flag flag,
						  bool assembleMatrix,
						  bool assembleVector)
  {
    FUNCNAME("ParallelProblemStatBase::buildAfterCoarsen()");

    TEST_EXIT_DBG(MeshDistributor::globalMeshDistributor != NULL)
      ("Should not happen!\n");

    MeshDistributor::globalMeshDistributor->checkMeshChange();
    ProblemStatSeq::buildAfterCoarsen(adaptInfo, flag, 
				      assembleMatrix, 
				      assembleVector);

    double vm, rss;
    processMemUsage(vm, rss);   
    vm /= 1024.0;
    rss /= 1024.0;
    
    MSG("My memory usage is VM = %.1f MB    RSS = %.1f MB\n", vm, rss);
    
    mpi::globalAdd(vm);
    mpi::globalAdd(rss);

    MSG("Overall memory usage is VM = %.1f MB    RSS = %.1f MB\n", vm, rss);
  }


  void ParallelProblemStatBase::initialize(Flag initFlag,
					   ProblemStatSeq *adoptProblem,
					   Flag adoptFlag)
  {
    ProblemStatSeq::initialize(initFlag, adoptProblem, adoptFlag);

    MeshDistributor::addProblemStatGlobal(this);

    meshDistributor = MeshDistributor::globalMeshDistributor;
  }

  
}
